#!/usr/bin/python3

import subprocess
import tempfile
import os
import re
import sys


def get_public_symbols_from_shared_object(prefix):
    """
    Find public (exported) symbols in a shared library (*.so) by calling
    the 'nm' command.
    :param prefix: CMAKE_INSTALL_PREFIX
    :return: a set of public symbols
    """
    so_path = os.path.join(prefix, "lib64", "libopenscap.so")
    nm_command = ["nm", "--demangle", "--dynamic", "--defined-only", so_path]
    nm_output_bytes = subprocess.check_output(nm_command)
    nm_output = nm_output_bytes.decode(encoding="utf-8")
    symbols = {line.split()[2] for line in nm_output.splitlines()}
    symbols = symbols.difference({"__bss_start", "_edata", "_end", "_fini",
                                  "_init"})
    return symbols


def get_public_headers(prefix):
    """
    It is useful to check if all the public headers are actually installed
    under $PREFIX/include/openscap.
    :param prefix: CMAKE_INSTALL_PREFIX
    :return: a set of public headers filepaths
    """
    public_headers = set()
    include_path = os.path.join(prefix, "include", "openscap")
    for root, dirs, files in os.walk(include_path):
        for f in files:
            full_path = os.path.join(root, f)
            public_headers.add(full_path)
    return public_headers


def get_public_symbols_from_header(header_path):
    """
    Parses a header file to find symbols marked by OSCAP_API macro.
    :param header_path: path to the header file
    :return: a list of public symbols in the header file
    """
    symbols = set()
    prototype_regex = re.compile(r"^[ \t]*OSCAP_API[^;]*;", re.MULTILINE)
    with open(header_path, "r") as header_file:
        content = header_file.read()
    prototypes = prototype_regex.findall(content)
    function_name_regex = re.compile(r"([a-zA-Z0-9_]+)\s*\(.*\)")
    data_name_regex = re.compile(r"\s+([a-zA-Z0-9_]+)\s*;")
    for p in prototypes:
        p = p.replace("\n", " ")
        match = function_name_regex.search(p)
        if not match:
            match = data_name_regex.search(p)
        if not match:
            print("Invalid prototype '%s'" % p, file=sys.stderr)
            continue
        symbol_name = match.group(1)
        symbols.add(symbol_name)
    return symbols


def get_public_symbols_from_headers(headers):
    """
    Find public (exported) symbols in header files by finding symbols
    marked as OSCAP_API.
    :param headers: a list of public headers file paths
    :return: a set of public symbols in public headers
    """
    symbols = set()
    for h in headers:
        symbols.update(get_public_symbols_from_header(h))
    return symbols


def _run_command(command):
    try:
        subprocess.run(command, check=True,
                       stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                       encoding="utf-8")
    except subprocess.CalledProcessError as e:
        print(e.output)
        print(e.stderr, file=sys.stderr)
        raise RuntimeError


def analyze_project_artifacts(src_dir):
    """
    Analyze the project and find all the symbols declared as public in
    public header files and all the symbols actually exported in built
    library. This is done on a real build and a real installation.
    The artifacts are installed in a temporary directory which is
    automatically removed when this function returns.
    :param src_dir: Path to project sources
    :return: tuple (header_symbols, so_symbols)
    """
    cwd = os.getcwd()
    build_dir = os.path.normpath(os.path.join(src_dir, "build"))
    build_dir_contents = os.listdir(build_dir)
    if ".gitkeep" in build_dir_contents:
        build_dir_contents.remove(".gitkeep")
    if build_dir_contents:
        print("Directory '%s' is not empty" % build_dir, file=sys.stderr)
        raise RuntimeError
    os.chdir(build_dir)
    with tempfile.TemporaryDirectory() as prefix:
        cmake_command = ["cmake",
                         "-DENABLE_PYTHON3=FALSE",
                         "-DENABLE_PERL=FALSE",
                         "-DCMAKE_INSTALL_PREFIX=" + prefix,
                         "-DENABLE_TESTS=FALSE",
                         ".."]

        _run_command(cmake_command)
        make_command = ["make"]
        _run_command(make_command)
        make_docs_command = ["make", "docs"]
        _run_command(make_docs_command)
        make_install_command = ["make", "install"]
        _run_command(make_install_command)
        public_headers = get_public_headers(prefix)
        so_symbols = get_public_symbols_from_shared_object(prefix)
        header_symbols = get_public_symbols_from_headers(public_headers)
    os.chdir(cwd)
    return header_symbols, so_symbols


def main():
    src_dir = os.path.join(os.getcwd(), "..")

    try:
        header_symbols, so_symbols = analyze_project_artifacts(src_dir)
    except RuntimeError as e:
        print("Could not analyze OpenSCAP public API", file=sys.stderr)
        sys.exit(1)

    print("Shared object symbols: %d" % len(so_symbols))
    print("Public header symbols: %d\n" % len(header_symbols))
    print()

    so_only = so_symbols.difference(header_symbols)
    if so_only:
        print("The following %d symbols are exported in binary, "
              "but are not present in public header files:" % len(so_only))
        for s in sorted(so_only):
            print(s)
        print()

    header_only = header_symbols.difference(so_symbols)
    if header_only:
        print("The following %d symbols are present in public header files, "
              "but are not exported in binary:" % len(header_only))
        for s in sorted(header_only):
            print(s)


if __name__ == "__main__":
    main()



# Copyright (C) 2015 Brent Baude <bbaude@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

''' Utilities for oscap-docker '''

from __future__ import print_function

import os
import tempfile
import subprocess
import platform
import shutil
from oscap_docker_python.get_cve_input import getInputCVE
import sys
import docker
import collections

try:
    from Atomic.mount import DockerMount
    from Atomic.mount import MountError
    import inspect

    if "mnt_mkdir" not in inspect.getargspec(DockerMount.__init__).args:
        sys.stderr.write(
            "\"Atomic.mount.DockerMount\" has been successfully imported but "
            "it doesn't support the mnt_mkdir argument. Please upgrade your "
            "Atomic installation to 1.4 or higher.\n"
        )
        sys.exit(1)

    # we only care about method names
    member_methods = [
        x[0] for x in
        inspect.getmembers(
            DockerMount, predicate=lambda member: inspect.isfunction(member) or inspect.ismethod(member)
        )
    ]

    if "_clean_temp_container_by_path" not in member_methods:
        sys.stderr.write(
            "\"Atomic.mount.DockerMount\" has been successfully imported but "
            "it doesn't have the _clean_temp_container_by_path method. Please "
            "upgrade your Atomic installation to 1.4 or higher.\n"
        )
        sys.exit(1)

except ImportError:
    sys.stderr.write(
        "Failed to import \"Atomic.mount.DockerMount\". It seems Atomic has "
        "not been installed.\n"
    )
    sys.exit(1)


class OscapError(Exception):
    ''' oscap Error'''
    pass


OscapResult = collections.namedtuple("OscapResult", ("returncode", "stdout", "stderr"))


class OscapHelpers(object):
    ''' oscap class full of helpers for scanning '''
    CPE = 'oval:org.open-scap.cpe.rhel:def:'
    DISTS = ["7", "6", "5"]

    def __init__(self, cve_input_dir):
        self.cve_input_dir = cve_input_dir

    @staticmethod
    def _mk_tmp_dir(tmp_dir):
        '''
        Creates a temporary directory and returns the whole
        path name
        '''
        tempfile.tempdir = tmp_dir
        return tempfile.mkdtemp()

    @staticmethod
    def _rm_tmp_dir(tmp_dir):
        '''
        Deletes the temporary directory created for the purposes
        of mount
        '''
        shutil.rmtree(tmp_dir)

    def _get_dist(self, chroot, target):
        '''
        Test the chroot and determine what RHEL dist it is; returns
        an integer representing the dist
        '''
        cpe_dict = '/usr/share/openscap/cpe/openscap-cpe-oval.xml'
        if not os.path.exists(cpe_dict):
            raise OscapError()
        for dist in self.DISTS:
            result = self.oscap_chroot(chroot, target, 'oval', 'eval',
                                       '--id', self.CPE + dist, cpe_dict,
                                       '2>&1', '>', '/dev/null')
            if "{0}{1}: true".format(self.CPE, dist) in result.stdout:
                return dist

    def _get_target_name(self, target):
        '''
        Determines if target is image or container. For images returns full
        image name if exists or image ID otherwise. For containers returns
        container name if exists or container ID otherwise.
        '''
        try:
            client = docker.APIClient()
        except AttributeError:
            client = docker.Client()
        try:
            image = client.inspect_image(target)
            if image["RepoTags"]:
                name = ", ".join(image["RepoTags"])
            else:
                name = image["Id"][len("sha256:"):][:10]
            return "docker-image://{}".format(name)
        except docker.errors.NotFound:
            try:
                container = client.inspect_container(target)
                if container["Name"]:
                    name = container["Name"].lstrip("/")
                else:
                    name = container["Id"][:10]
                return "docker-container://{}".format(name)
            except docker.errors.NotFound:
                return "unknown"

    def oscap_chroot(self, chroot_path, target, *oscap_args):
        '''
        Wrapper function for executing oscap in a subprocess
        '''

        os.environ["OSCAP_PROBE_ARCHITECTURE"] = platform.processor()
        os.environ["OSCAP_PROBE_ROOT"] = os.path.join(chroot_path)
        os.environ["OSCAP_PROBE_OS_NAME"] = platform.system()
        os.environ["OSCAP_PROBE_OS_VERSION"] = platform.release()
        os.environ["OSCAP_EVALUATION_TARGET"] = self._get_target_name(target)
        cmd = ['oscap'] + [x for x in oscap_args]
        oscap_process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        oscap_stdout, oscap_stderr = oscap_process.communicate()
        return OscapResult(oscap_process.returncode,
                           oscap_stdout.decode("utf-8"), oscap_stderr.decode("utf-8"))

    def _scan_cve(self, chroot, target, dist, scan_args):
        '''
        Scan a chroot for cves
        '''
        cve_input = getInputCVE.dist_cve_name.format(dist)
        tmp_tuple = ('oval', 'eval') + tuple(scan_args) + \
            (os.path.join(self.cve_input_dir, cve_input),)
        return self.oscap_chroot(chroot, target, *tmp_tuple)

    def _scan(self, chroot, target, scan_args):
        '''
        Scan a container or image
        '''
        tmp_tuple = tuple(scan_args)
        return self.oscap_chroot(chroot, target, *tmp_tuple)

    def resolve_image(self, image):
        '''
        Given an image or container name, uuid, or partial, return the
        uuid or iid or False if cannot be identified
        '''
        # TODO
        pass

    def _cleanup_by_path(self, path, DM):
        '''
        Cleans up the mounted chroot by umounting it and
        removing the temporary directory
        '''
        # Sometimes when this def is called, path will have 'rootfs'
        # appended.  If it does, strip it and proceed
        _no_rootfs = path
        if os.path.basename(path) == 'rootfs':
            _no_rootfs = os.path.dirname(path)

        # umount chroot
        DM.unmount_path(_no_rootfs)

        # clean up temporary container
        DM._clean_temp_container_by_path(_no_rootfs)
        os.rmdir(_no_rootfs)


def mount_image_filesystem():
    _tmp_mnt_dir = DM.mount(image)


class OscapScan(object):
    def __init__(self, tmp_dir=tempfile.gettempdir(), mnt_dir=None,
                 hours_old=2):
        self.tmp_dir = tmp_dir
        self.helper = OscapHelpers(tmp_dir)
        self.mnt_dir = mnt_dir
        self.hours_old = hours_old

    def _ensure_mnt_dir(self):
        '''
        Ensure existing temporary directory
        '''
        if self.mnt_dir is None:
            return tempfile.mkdtemp()
        else:
            return self.mnt_dir

    def _remove_mnt_dir(self, mnt_dir):
        '''
        Remove temporary directory, but only if the directory was not
        passed through __init__
        '''
        if self.mnt_dir is None:
            os.rmdir(mnt_dir)

    def _find_chroot_path(self, mnt_dir):
        '''
        Remember actual mounted fs in 'rootfs' for devicemapper
        '''
        rootfs_path = os.path.join(mnt_dir, 'rootfs')
        if os.path.exists(rootfs_path):
            chroot = rootfs_path
        else:
            chroot = mnt_dir
        return chroot

    def scan_cve(self, image, scan_args):
        '''
        Wrapper function for scanning a container or image
        '''

        mnt_dir = self._ensure_mnt_dir()

        # Mount the temporary image/container to the dir
        DM = DockerMount(mnt_dir, mnt_mkdir=True)
        try:
            _tmp_mnt_dir = DM.mount(image)
        except MountError as e:
            sys.stderr.write(str(e) + "\n")
            return None

        try:
            chroot = self._find_chroot_path(_tmp_mnt_dir)

            # Figure out which RHEL dist is in the chroot
            dist = self.helper._get_dist(chroot, image)

            if dist is None:
                sys.stderr.write("{0} is not based on RHEL\n".format(image))
                return None

            # Fetch the CVE input data for the dist
            fetch = getInputCVE(self.tmp_dir)
            fetch._fetch_single(dist)

            # Scan the chroot
            scan_result = self.helper._scan_cve(chroot, image, dist, scan_args)
            print(scan_result.stdout)
            print(scan_result.stderr, file=sys.stderr)

        finally:
            # Clean up
            self.helper._cleanup_by_path(_tmp_mnt_dir, DM)
            self._remove_mnt_dir(mnt_dir)

        return scan_result.returncode

    def scan(self, image, scan_args):
        '''
        Wrapper function for basic security scans using
        openscap
        '''

        mnt_dir = self._ensure_mnt_dir()

        # Mount the temporary image/container to the dir
        DM = DockerMount(mnt_dir, mnt_mkdir=True)
        try:
            _tmp_mnt_dir = DM.mount(image)
        except MountError as e:
            sys.stderr.write(str(e) + "\n")
            return None

        try:
            chroot = self._find_chroot_path(_tmp_mnt_dir)

            # Scan the chroot
            scan_result = self.helper._scan(chroot, image, scan_args)
            print(scan_result.stdout)
            print(scan_result.stderr, file=sys.stderr)

        finally:
            # Clean up
            self.helper._cleanup_by_path(_tmp_mnt_dir, DM)
            self._remove_mnt_dir(mnt_dir)

        return scan_result.returncode



# Copyright (C) 2015 Brent Baude <bbaude@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import urllib.parse
import urllib.request
from os.path import join, exists
from os import stat, utime
from sys import stderr
import datetime


class getInputCVE(object):
    '''
    Class to obtain the CVE data provided by RH used to scan
    for CVEs using openscap
    '''

    hdr = {'User-agent': 'Mozilla/5.0'}
    hdr2 = [('User-agent', 'Mozilla/5.0')]
    url = "https://www.redhat.com/security/data/oval/"
    dist_cve_name = "com.redhat.rhsa-RHEL{0}.xml.bz2"
    dists = [5, 6, 7]
    remote_pattern = '%a, %d %b %Y %H:%M:%S %Z'

    def __init__(self, fs_dest, DEBUG=False):
        ''' Simple init declaration '''
        self.dest = fs_dest
        self.DEBUG = DEBUG

    def _fetch_single(self, dist):
        '''
        Given a distribution number (i.e. 7), it will fetch the
        distribution specific data file if upstream has a newer
        input file.  Returns the path of file.
        '''
        cve_file = self.dist_cve_name.format(dist)
        dest_file = join(self.dest, cve_file)
        dist_url = urllib.parse.urljoin(self.url, cve_file)
        if self._is_cache_same(dest_file, dist_url):
            return dest_file

        _url = urllib.request.Request(dist_url, headers=self.hdr)
        # TODO
        # When dist specific files are available in bz form, some
        # of this logic may need to change
        try:
            resp = urllib.request.urlopen(_url)

        except Exception as url_error:
            raise Exception("Unable to fetch CVE inputs due to {0}"
                            .format(url_error))

        fh = open(dest_file, "wb")
        fh.write(resp.read())
        fh.close()

        # Correct Last-Modified timestamp
        headers = self._parse_http_headers(resp.info())
        resp.close()
        try:
            remote_ts = headers['last-modified']
            epoch = datetime.datetime.utcfromtimestamp(0)
            remote_dt = datetime.datetime.strptime(remote_ts, self.remote_pattern)
            seconds_epoch = (remote_dt - epoch).total_seconds()
            utime(dest_file, (seconds_epoch, seconds_epoch))
        except KeyError:
            self._print_no_last_modified_warning(dist_url)

        return dest_file

    def _print_no_last_modified_warning(self, url):
        if self.DEBUG:
            stderr.write("Warning: Response header of HTTP doesn't contain " \
                         "\"last-modified\" field. Cannot determine version" \
                         " of remote file \"{0}\"\n".format(url))

    def _parse_http_headers(self, http_headers):
        '''
        Returns dictionary containing HTTP headers with lowercase keys
        '''

        headers_dict = dict(http_headers)
        return dict( (key.lower(), value) for key, value in headers_dict.items() )

    def _is_cache_same(self, dest_file, dist_url):
        '''
        Checks if the local cache version and the upstream
        version is the same or not.  If they are the same,
        returns True; else False.
        '''

        if not exists(dest_file):
            if self.DEBUG:
                stderr.write("No file in cache, fetching {0}\n".format(dest_file))
            return False

        opener = urllib.request.build_opener()
        # Add the header
        opener.addheaders = self.hdr2
        # Grab the header
        try:
            res = opener.open(HeadRequest(dist_url))
            headers = self._parse_http_headers(res.info())
            res.close()
            remote_ts = headers['last-modified']

        except urllib.request.HTTPError as http_error:
            if self.DEBUG:
                stderr.write("Cannot send HTTP HEAD request to get \"last-modified\"" \
                             " attribute of remote content file.\n{0} - {1}\n"
                             .format(http_error.code, http_error.reason))
            return False

        except KeyError:
            self._print_no_last_modified_warning(dist_url)
            return False

        # The remote's datetime
        remote_dt = datetime.datetime.strptime(remote_ts, self.remote_pattern)
        # Get the locals datetime from the file's mtime, converted to UTC
        local_dt = datetime.datetime.utcfromtimestamp((stat(dest_file))
                                                      .st_mtime)

        # Giving a two second comfort zone
        # Else we declare they are different
        if (remote_dt - local_dt).seconds > 2:
            if self.DEBUG:
                stderr.write("Had a local file {0} " \
                      "but it wasn't new enough\n".format(dest_file))
            return False
        if self.DEBUG:
            stderr.write("File {0} is same as upstream\n".format(dest_file))

        return True

    def fetch_dist_data(self):
        '''
        Fetches all the the distribution specific data used for
        input with openscap cve scanning and returns a list
        of those files.
        '''
        cve_files = []
        for dist in self.dists:
            cve_files.append(self._fetch_single(dist))
        return cve_files


class HeadRequest(urllib.request.Request):
    def get_method(self):
        return 'HEAD'



import argparse as ap

import github


def update_parser_with_common_stuff(parser):
    parser.add_argument("--owner", default="openscap")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("--auth-token")


def get_milestone(repo, name):
    milestones = repo.get_milestones()
    matches = [m for m in milestones if m.title == name]
    assert len(matches) <= 1, \
        f"Expected to find at most one milestone {name}, found {len(matches)}"
    if len(matches) == 0:
        return None
    else:
        return matches[0]


def get_github(args):
    if args.auth_token is not None:
        return github.Github(args.auth_token)
    else:
        assert 0, "Credentials were not supplied"


def get_repo(github, owner):
    repo = github.get_repo(f"{owner}/openscap")
    return repo


def version_type(string):
    components = string.split(".")
    shortest_component_len = min([len(x) for x in components])
    if len(components) != 3 or shortest_component_len == 0:
        msg = (
            "Expected version number of form X.Y.Z, where X, Y, Z are strings. "
            f"Got: '{string}'"
        )
        raise ap.ArgumentTypeError(msg)
    return string



#!/usr/bin/python3

#
# Copyright 2011 Red Hat Inc., Durham, North Carolina.
# All Rights Reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors:
#      "Martin Preisler" <mpreisle@redhat.com>
#

import argparse

parser = argparse.ArgumentParser(description = "Performs checks by given scripts and returns the result")
parser.add_argument("globs", metavar = "F", type = str, nargs = "+",
                   help = "File or glob of the check script to process")

args = parser.parse_args()

import glob
files = []
for g in args.globs:
    gfiles = glob.glob(g)
    files.extend(gfiles)
    if len(gfiles) == 0:
        print("No file matches '%s'" % (g))

import os

xccdf_result_map = dict()
xccdf_result_map["XCCDF_RESULT_PASS"] = 101
xccdf_result_map["XCCDF_RESULT_FAIL"] = 102
xccdf_result_map["XCCDF_RESULT_ERROR"] = 103
xccdf_result_map["XCCDF_RESULT_UNKNOWN"] = 104
xccdf_result_map["XCCDF_RESULT_NOT_APPLICABLE"] = 105
xccdf_result_map["XCCDF_RESULT_NOT_CHECKED"] = 106
xccdf_result_map["XCCDF_RESULT_NOT_SELECTED"] = 107
xccdf_result_map["XCCDF_RESULT_INFORMATIONAL"] = 108
xccdf_result_map["XCCDF_RESULT_FIXED"] = 109

xccdf_reverse_result_map = dict()
for key, value in xccdf_result_map.items():
    xccdf_reverse_result_map[value] = key

# set all XCCDF result types as environment variables
# (this is especially useful for bash scripts)
for key, value in xccdf_result_map.items():
    os.environ[key] = str(value)

import subprocess

for file_path in files:
    result = subprocess.call(["./%s" % (file_path)])
    if result in xccdf_reverse_result_map:
        result_text = xccdf_reverse_result_map[result]
    else:
        result_text = "Unknown exit code %i" % (result)
    print("Result of '%s' is %s" % (file_path, result_text))




from __future__ import print_function

import argparse as ap
from xml.etree import ElementTree


NAMESPACES = dict(
    xccdf_ns="http://scap.nist.gov/schema/scap/source/1.2",
    profile_ns="http://checklists.nist.gov/xccdf/1.2",
)


class RuleResult(object):
    def __init__(self, element):
        self.idref = element.attrib["idref"]
        self.weight = element.attrib["weight"]

        result_element = element.find("{%s}result" % NAMESPACES["profile_ns"])
        self.result_text = result_element.text

        check_element = element.find("{%s}check" % NAMESPACES["profile_ns"])
        self.check_system = check_element.attrib["system"]
        self.check_system = check_element.attrib["system"]

        check_content_ref_element = check_element.find(
            "{%s}check-content-ref" % NAMESPACES["profile_ns"])
        self.content_ref_href = check_content_ref_element.attrib["href"]
        self.content_ref_name = check_content_ref_element.attrib["name"]

        check_export_element = check_element.find(
            "{%s}check-export" % NAMESPACES["profile_ns"])
        self.check_export_name = None
        self.check_export_value_id = None
        if check_export_element is not None:
            self.check_export_name = check_export_element.attrib["export-name"]
            self.check_export_value_id = check_export_element.attrib["value-id"]

    def __hash__(self):
        return hash(frozenset(self.__dict__.items()))

    def __eq__(self, rhs):
        return hash(self) == hash(rhs)


def fname_to_etree(fname):
    input_tree = ElementTree.parse(fname)
    return input_tree


def get_rule_results_from_etree(tree):
    xpath_expr =  ".//{%s}rule-result" % NAMESPACES["profile_ns"]
    xccdfs = tree.findall(xpath_expr)
    return xccdfs


def extract_results_from_file(fname):
    tree = fname_to_etree(fname)
    results = get_rule_results_from_etree(tree)
    results = {RuleResult(r) for r in results}
    return results


def make_parser():
    parser = ap.ArgumentParser()
    parser.add_argument("first")
    parser.add_argument("second")
    return parser


if __name__ == "__main__":
    parser = make_parser()
    args = parser.parse_args()
    first_results = extract_results_from_file(args.first)
    second_results = extract_results_from_file(args.second)
    assert len(first_results) == len(second_results), \
        "There is not the same number of results in the two input files"
    for result in first_results:
        assert result in second_results, \
            "There is a result in the first file that isn't in the second file"
